import sqlite3
import logging
import re

from .setup_db import create_db, get_db_path

class DB:
    def __init__(self)->None:
        create_db()

        self.conn = sqlite3.connect(get_db_path())
        self.cursor = self.conn.cursor()

    def get_location_from_plate_id(self, plate_id: str) -> int:
        self.cursor.execute("SELECT Location FROM Plates WHERE PlateId = ?", (plate_id,))
        result = self.cursor.fetchone()
        return result[0] if result else None

    def get_owner_from_plate_id(self, plate_id: str) -> str:
        self.cursor.execute("SELECT Owner FROM Plates WHERE PlateId = ?", (plate_id,))
        result = self.cursor.fetchone()
        return result[0] if result else None

    def get_all_plate_ids_from_owner(self, owner: str) -> list:
        self.cursor.execute("SELECT PlateId FROM Plates WHERE Owner = ?", (owner,))
        return [row[0] for row in self.cursor.fetchall()]

    def get_all_plate_ids_from_location(self, location: int) -> list:
        self.cursor.execute("SELECT PlateId FROM Plates WHERE Location = ?", (location,))
        return [row[0] for row in self.cursor.fetchall()]

    def get_role_from_plate_id(self, plate_id: str) -> int:
        self.cursor.execute(
            "SELECT Role FROM Slots WHERE CurrentPlateId = ? OR AssignedPlatePlateId = ?",
            (plate_id, plate_id)
        )
        result = self.cursor.fetchone()
        return result[0] if result else None

    def get_current_plate_id_from_slot_stacker(self, slot: int, stacker: int) -> str:
        self.cursor.execute(
            "SELECT CurrentPlateId FROM Slots WHERE SlotNumber = ? AND Stacker = ?",
            (slot, stacker)
        )
        result = self.cursor.fetchone()
        return result[0] if result else None

    def get_assigned_plate_id_from_slot_stacker(self, slot: int, stacker: int) -> str:
        self.cursor.execute(
            "SELECT AssignedPlatePlateId FROM Slots WHERE SlotNumber = ? AND Stacker = ?",
            (slot, stacker)
        )
        result = self.cursor.fetchone()
        return result[0] if result else None

    def get_slot_stacker_from_assigned_plate_id(self, plate_id: str) -> tuple:
        self.cursor.execute(
            "SELECT SlotNumber, Stacker FROM Slots WHERE AssignedPlatePlateId = ?",
            (plate_id,)
        )
        return self.cursor.fetchone()

    def get_slot_stacker_from_current_plate_id(self, plate_id: str) -> tuple:
        self.cursor.execute(
            "SELECT SlotNumber, Stacker FROM Slots WHERE CurrentPlateId = ?",
            (plate_id,)
        )
        return self.cursor.fetchone()

    def get_tables(self)->list:
        self.cursor.execute("SELECT name FROM sqlite_master WHERE type='table';")
        return [row[0] for row in self.cursor.fetchall()]

    def get_columns_from_table(self, table: str)->list:
        self.cursor.execute(f"PRAGMA table_info({table})")
        return [row[1] for row in self.cursor.fetchall()]

    def get_table_data(self, table)->list[tuple[any]]:
        self.cursor.execute(f"SELECT * FROM {table}")
        return self.cursor.fetchall()

    def get_all_slots_from_role_stacker(self, role: int, stacker: int) -> list:
        self.cursor.execute(
            "SELECT SlotNumber FROM Slots WHERE Role = ? AND Stacker = ?",
            (role, stacker)
        )
        return [row[0] for row in self.cursor.fetchall()]

    #Well methods
    def get_well_id_from_plate_id_location(self, plate_id: str, location: str) -> str:
        try:
            self.cursor.execute(
                "SELECT WellId FROM Wells WHERE plateID = ? AND location = ?",
                (plate_id, location)
            )
            row = self.cursor.fetchone()
            return row[0] if row else None
        except sqlite3.Error as e:
            logging.error(f"Error getting WellId for {location} on plate {plate_id}: {e}")
            return None

    def get_media_from_location_plate_id(self, plate_id: str, location: str) -> dict:
        try:
            self.cursor.execute(
                "SELECT is_media, mediachange FROM Wells WHERE plateID = ? AND location = ?",
                (plate_id, location)
            )
            row = self.cursor.fetchone()
            if row:
                return {
                    "is_media": bool(row[0]),
                    "mediachange": bool(row[1])
                }
            return {}
        except sqlite3.Error as e:
            logging.error(f"Error fetching media info for {location} on plate {plate_id}: {e}")
            return {}

    def get_well_row(self, plate_id: str, location: str) -> dict:
        try:
            self.cursor.execute(
                "SELECT * FROM Wells WHERE plateID = ? AND location = ?",
                (plate_id, location)
            )
            row = self.cursor.fetchone()
            if row:
                columns = [desc[0] for desc in self.cursor.description]
                return dict(zip(columns, row))
            return {}
        except sqlite3.Error as e:
            logging.error(f"Error fetching row for {location} on plate {plate_id}: {e}")
            return {}

    def set_role_from_plate_id(self, plate_id: str, role: int):
        try:
            self.cursor.execute(
                "UPDATE Slots SET Role = ? WHERE CurrentPlateId = ? OR AssignedPlatePlateId = ?",
                (role, plate_id, plate_id)
            )
            self.conn.commit()
        except sqlite3.Error as e:
            logging.error(f"Error setting role for plate {plate_id}: {e}")

    def set_role_from_slot_stacker(self, slot: int, stacker: int, role: int):
        try:
            self.cursor.execute(
                "UPDATE Slots SET Role = ? WHERE SlotNumber = ? AND Stacker = ?",
                (role, slot, stacker)
            )
            self.conn.commit()
        except sqlite3.Error as e:
            logging.error(f"Error setting role for slot {slot}, stacker {stacker}: {e}")

    def set_current_plate_id_from_slot_stacker(self, slot: int, stacker: int, plate_id: str):
        try:
            self.cursor.execute(
                "UPDATE Slots SET CurrentPlateId = ? WHERE SlotNumber = ? AND Stacker = ?",
                (plate_id, slot, stacker)
            )
            self.conn.commit()
        except sqlite3.Error as e:
            logging.error(f"Error setting current plate {plate_id} for slot {slot}, stacker {stacker}: {e}")

    def set_assigned_plate_id_from_slot_stacker(self, slot: int, stacker: int, plate_id: str):
        try:
            self.cursor.execute(
                "UPDATE Slots SET AssignedPlatePlateId = ? WHERE SlotNumber = ? AND Stacker = ?",
                (plate_id, slot, stacker)
            )
            self.conn.commit()
        except sqlite3.Error as e:
            logging.error(f"Error assigning plate {plate_id} to slot {slot}, stacker {stacker}: {e}")

    def set_owner_from_plate_id(self, plate_id: str, owner: str):
        try:
            self.cursor.execute(
                "UPDATE Plates SET Owner = ? WHERE PlateId = ?",
                (owner, plate_id)
            )
            self.conn.commit()
        except sqlite3.Error as e:
            logging.error(f"Error setting owner {owner} for plate {plate_id}: {e}")

    def set_location_from_plate_id(self, plate_id: str, location: int):
        try:
            self.cursor.execute(
                "UPDATE Plates SET Location = ? WHERE PlateId = ?",
                (location, plate_id)
            )
            self.conn.commit()
        except sqlite3.Error as e:
            logging.error(f"Error setting location {location} for plate {plate_id}: {e}")

    def register_slot(self, slot: int, stacker: int, role: int, occupied: int, assigned_id: str, current_id: str):
        try:
            self.cursor.execute(
                """ INSERT INTO Slots (Stacker, SlotNumber, Role, Occupied, AssignedPlatePlateId, CurrentPlateId)
                    VALUES (?, ?, ?, ?, ?, ?)""",
                (stacker, slot, role, occupied, assigned_id, current_id)
            )
            self.conn.commit()
        except sqlite3.IntegrityError:
            logging.warning(f"Slot ({stacker}, {slot}) already exists – skipping insert.")
        except sqlite3.Error as e:
            logging.error(f"Error inserting slot ({stacker}, {slot}): {e}")

    def register_plate(self, plate_id: str, owner: str, location: int, wells: int = 48, format: str = "6x8"):

        if not re.fullmatch(r"\d+x\d+", format):
            raise ValueError(f"Invalid format '{format}'. Expected format like '6x8'.")

        try:
            self.cursor.execute(
                "INSERT INTO Plates (PlateId, Owner, Location, Wells, Format) VALUES (?, ?, ?, ?, ?)",
                (plate_id, owner, location, wells, format)
            )
            self.conn.commit()
        except sqlite3.IntegrityError:
            logging.warning(f"Plate {plate_id} already exists – skipping insert.")
        except sqlite3.Error as e:
            logging.error(f"Error inserting plate {plate_id}: {e}")

    def register_plate_with_wells(self, plate_id: str, owner: str, location: int, wells: int = 48, format: str = "6x8"):
        try:
            self.register_plate(plate_id = plate_id, owner = owner, location=location, wells = wells, format = format)

            format_rows, format_cols = map(int, format.lower().split("x"))

            for row in range(format_rows):
                row_letter = chr(ord('A') + row)  # A, B, C, ...
                for col in range(1, format_cols + 1):
                    location_label = f"{row_letter}{col}"  # z. B. A1, B3, F8
                    self.register_well(plate_id=plate_id, location=location_label)

            logging.info(f"Plate {plate_id} with wells created successfully.")

        except sqlite3.IntegrityError:
            logging.warning(f"Plate {plate_id} already exists – skipping insert.")
        except sqlite3.Error as e:
            logging.error(f"Error inserting plate {plate_id}: {e}")
        except Exception as e:
            logging.error(f"Unexpected Error on {plate_id}: {e}")

    def update_well(self, plate_id: str, location: str, is_media: bool = False, mediachange: bool = False):
        try:
            self.cursor.execute(
                "UPDATE Wells SET is_media = ?, mediachange = ? WHERE plateID = ? AND location = ?",
                (int(is_media), int(mediachange), plate_id, location)
            )
            self.conn.commit()
        except sqlite3.Error as e:
            logging.error(f"Error updating well at {location} on plate {plate_id}: {e}")

    def register_well(self, plate_id: str, location: str, mediachange: bool = False, is_media: bool = False):
        try:
            self.cursor.execute(
                "INSERT INTO Wells (plateID, location, mediachange, isMedia) VALUES (?, ?, ?, ?)",
                (plate_id, location, int(mediachange), int(is_media))
            )
            self.conn.commit()
        except sqlite3.IntegrityError:
            logging.warning(f"Well {location} for plate {plate_id} already exists – skipping insert.")
        except sqlite3.Error as e:
            logging.error(f"Error inserting well {location} for plate {plate_id}: {e}")


    def write_plate_action(self, time_stamp: str, plate_id: str, action: str):
        try:
            self.cursor.execute(
                "INSERT INTO PlateActions (Timestamp, PlateId, Action) VALUES (?, ?, ?)",
                (time_stamp, plate_id, action)
            )
            self.conn.commit()
        except sqlite3.IntegrityError:
            logging.warning(f"Action at {time_stamp} already exists – skipping.")
        except sqlite3.Error as e:
            logging.error(f"Error inserting plate action at {time_stamp}: {e}")


import sqlite3
import os
from pathlib import Path
import logging

def get_config_dir()->Path:
    return Path('C:/ProgramData/Incubator')

def get_db_path()->Path:
    return get_config_dir().joinpath("Incubator.db")

def create_db_dir():
    config_path = get_config_dir()
    try:
        if not config_path.exists():
            os.mkdir(config_path)
            logging.info(("Created Directory:", config_path))
    except Exception:
        logging.error((f"Could not create", config_path))


def create_db():
    config_path = get_config_dir()
    db_path = get_db_path()
    try:
        if not config_path.exists():
            create_db_dir()

        if db_path.exists():
            return

        #connetct to db/create empty db
        conn = sqlite3.connect(db_path)
        cursor = conn.cursor()

        #Create table PlateActions
        cursor.execute("""  CREATE TABLE "PlateActions" (
                            "Timestamp" TEXT NOT NULL CONSTRAINT "PK_PlateActions" PRIMARY KEY,
                            "PlateId" TEXT NULL,
                            "Action" TEXT NULL,
                            CONSTRAINT "FK_PlateActions_Plates_PlateId" FOREIGN KEY ("PlateId") REFERENCES "Plates" ("PlateId") ON DELETE RESTRICT)""")

        #create table Plates
        cursor.execute("""  CREATE TABLE "Plates" (
                            "PlateId" TEXT NOT NULL CONSTRAINT "PK_Plates" PRIMARY KEY,
                            "Owner" TEXT NULL,
                            "Location" INTEGER NOT NULL,
                            "Wells" INTEGER NOT NULL,
                            "Format" TEXT NOT NULL DEFAULT '6x8')""")

        try:
            cursor.execute("""  CREATE TABLE "Wells" (
                                "WellId" INTEGER PRIMARY KEY AUTOINCREMENT,
                                "PlateId" TEXT NOT NULL,
                                "location" TEXT NOT NULL,
                                "mediachange" BOOLEAN NOT NULL DEFAULT 0,
                                "isMedia" BOOLEAN NOT NULL DEFAULT 0,
                                FOREIGN KEY ("PlateId") REFERENCES Plates("PlateId") ON DELETE CASCADE,
                                UNIQUE ("plateID", "location"))""")
        except Exception as e:
            print("Wells error: ", e)

        #create table slots
        cursor.execute("""  CREATE TABLE "Slots" (
                            "SlotNumber" INTEGER NOT NULL,
                            "Stacker" INTEGER NOT NULL,
                            "Role" INTEGER NOT NULL,
                            "Occupied" INTEGER NOT NULL,
                            "AssignedPlatePlateId" TEXT NULL,
                            "CurrentPlateId" TEXT NULL,
                            CONSTRAINT "PK_Slots" PRIMARY KEY ("Stacker", "SlotNumber"),
                            CONSTRAINT "FK_Slots_Plates_AssignedPlatePlateId" FOREIGN KEY ("AssignedPlatePlateId") REFERENCES "Plates" ("PlateId") ON DELETE RESTRICT,
                            CONSTRAINT "FK_Slots_Plates_CurrentPlateId" FOREIGN KEY ("CurrentPlateId") REFERENCES "Plates" ("PlateId") ON DELETE RESTRICT) """)

        #create index IX_PlateActions_PlateId
        cursor.execute("""  CREATE INDEX "IX_PlateActions_PlateId" ON "PlateActions" ("PlateId")""")

        #create Index Slots_AssignedPlatePlateId
        cursor.execute("""  CREATE UNIQUE INDEX "IX_Slots_AssignedPlatePlateId" ON "Slots" ("AssignedPlatePlateId")""")

        #create Index Slots_CurrentPlateID
        cursor.execute("""  CREATE INDEX "IX_Slots_CurrentPlateId" ON "Slots" ("CurrentPlateId")""")

        conn.close()

    except sqlite3.Error as e:
        logging.error(msg =("Sqlite Error occurred", e))

create_db()


import re

import ttkbootstrap as ttk
from ttkbootstrap.constants import *
from .db import DB

class GUI:
    def __init__(self, master: str | None = None):
        """
        Parameters
        master : ttk.Window | None
            optional, depends on the use case of the GUI,
            can be integrated into another ttk.Window as a TopLevel or as a standalone GUI
        """
        self.root = ttk.Toplevel(master) if master else ttk.Window(title="Incubator Data Base", themename="vapor")
        self.root.geometry("1200x800")
        self.db = DB()
        self.tables = self.db.get_tables()
        print("Tabellen:", self.tables)

        self.root.columnconfigure(0, weight=1)
        self.root.columnconfigure(1, weight=1)
        self.root.columnconfigure(2, weight=1)
        self.root.columnconfigure(3, weight=1)
        self.root.rowconfigure(0, weight=0)
        self.root.rowconfigure(1, weight=0)
        self.root.rowconfigure(2, weight=12)

        style = ttk.Style()
        style.configure("Default.TButton")

        self.bool_slot = False
        self.bool_plate_actions = False
        self.bool_plates = False
        self.bool_wells = False

        self.button_slots = ttk.Button(text="Slots", command=self.callback_slots)
        self.button_plates = ttk.Button(text="Plates", command = self.callback_plates, bootstyle = "light")
        self.button_plate_actions = ttk.Button(text="PlateActions", command = self.callback_plate_actions)
        self.button_wells = ttk.Button(text = "Wells", command = self.callback_wells)

        self.button_slots.grid(column=0, row=0, sticky='nsew', ipady=12)
        self.button_plates.grid(column=1, row= 0, sticky='nsew', ipady=12)
        self.button_plate_actions.grid(column=2, row=0, sticky='nsew', ipady=12)
        self.button_wells.grid(column = 3, row = 0, sticky="nsew", ipady=12)

        self.tree_plates = ttk.Treeview(self.root)
        self.tree_plate_actions = ttk.Treeview(self.root)
        self.tree_slots = ttk.Treeview(self.root)
        self.tree_wells = ttk.Treeview(self.root)

        self.frame_entry = ttk.Frame(self.root)
        self.frame_entry.grid(row = 1, column=0, columnspan=4, sticky="nsew")
        self.entry_widgets = []#
        self.save_button = ttk.Button(self.root, text="Enter", command=self.save_entry, bootstyle = "success")
        self.save_button.grid(row=0, column=3, sticky="se", ipadx = 5, ipady = 5)
        self.update_entry_fields(self.db.get_columns_from_table("plates"))

        self.button_list = [self.button_wells, self.button_plates, self.button_plate_actions, self.button_slots]

        self.init_tree_plates()
        self.init_tree_plate_actions()
        self.init_tree_slots()
        self.init_tree_wells()
        self.insert_data()
        self.current_tree = self.tree_plates
        self.current_tree.grid()


    def get_root(self)->ttk.Window:
        return self.root

    def init_tree_plates(self):
        columns = self.db.get_columns_from_table(self.tables[1])
        self.tree_plates["columns"] = columns

        self.tree_plates.column("#0", width=0, stretch=False)
        self.tree_plates.heading("#0", text="")

        for i in columns:
            self.tree_plates.column(i, anchor=CENTER)
            self.tree_plates.heading(i, text=i, anchor=CENTER)

        self.tree_plates.grid(row=2, column=0, columnspan=4, sticky='nsew', pady=5)
        self.tree_plates.grid_remove()

    def init_tree_slots(self):
        columns = self.db.get_columns_from_table(self.tables[4])
        self.tree_slots["columns"] = columns

        self.tree_slots.column("#0", width=0, stretch=False)
        self.tree_slots.heading("#0", text="")

        for i in columns:
            self.tree_slots.column(i, anchor=CENTER)
            self.tree_slots.heading(i, text = i, anchor=CENTER)

        self.tree_slots.grid(row=2, column=0, columnspan=4, sticky='nsew', pady=5)
        self.tree_slots.grid_remove()

    def init_tree_plate_actions(self):
        columns = self.db.get_columns_from_table(self.tables[0])
        self.tree_plate_actions["columns"] = columns

        self.tree_plate_actions.column("#0", width=0, stretch=False)
        self.tree_plate_actions.heading("#0", text="")

        for i in columns:
            self.tree_plate_actions.column(i, anchor=CENTER)
            self.tree_plate_actions.heading(i, text=i, anchor=CENTER)

        self.tree_plate_actions.grid(row=2, column=0, columnspan=4, sticky='nsew', pady=5)
        self.tree_plate_actions.grid_remove()

    def init_tree_wells(self):
        columns = self.db.get_columns_from_table(self.tables[2])

        self.tree_wells["columns"] = columns
        self.tree_wells.column("#0", width=0, stretch=False)
        self.tree_wells.heading("#0", text="")

        for i in columns:
            self.tree_wells.column(i, anchor=CENTER)
            self.tree_wells.heading(i, text=i, anchor=CENTER)

        self.tree_wells.grid(row=2, column=0, columnspan=4, sticky='nsew', pady=5)
        self.tree_wells.grid_remove()

    def insert_data(self):
        table_data_plate_action: list[tuple] = self.db.get_table_data(self.tables[0])
        table_data_plates: list[tuple] = self.db.get_table_data(self.tables[1])
        table_data_slots: list[tuple] = self.db.get_table_data(self.tables[4])
        table_data_wells: list[tuple] = self.db.get_table_data(self.tables[2])

        for i in range(len(table_data_plate_action)):
            self.tree_plate_actions.insert(parent="", index = 'end', iid=i, text="Parent", values=table_data_plate_action[i])

        for i in range(len(table_data_plates)):
            self.tree_plates.insert(parent="", index = 'end', iid=i, text="Parent", values=table_data_plates[i])

        for i in range(len(table_data_slots)):
            self.tree_slots.insert(parent="", index = 'end', iid=i, text="Parent", values=table_data_slots[i])

        for i in range(len(table_data_wells)):
            self.tree_wells.insert(parent="", index = 'end', iid=i, text="Parent", values=table_data_wells[i])


    def callback_slots(self):
        if self.bool_slot:
            return

        self.bool_plates = False
        self.bool_plate_actions = False
        self.bool_slot = True
        self.current_tree.grid_remove()
        self.current_tree = self.tree_slots
        self.current_tree.grid()
        for i in range(4):
            self.reset_button_state(self.button_list[i])
        self.button_slots.configure(bootstyle="light")

        self.update_entry_fields(self.db.get_columns_from_table(self.tables[4]))

        #update data
        self.tree_slots.delete(*self.tree_slots.get_children())
        table_data = self.db.get_table_data("Slots")
        for row in table_data:
            self.tree_slots.insert("", "end", values=row)

    def callback_plates(self):
        if self.bool_plates:
            return

        self.bool_slot = False
        self.bool_plate_actions = False
        self.bool_plates = True
        self.current_tree.grid_remove()
        self.current_tree = self.tree_plates
        self.current_tree.grid()
        for i in range(4):
            self.reset_button_state(self.button_list[i])
        self.button_plates.configure(bootstyle="light")

        self.update_entry_fields(self.db.get_columns_from_table(self.tables[1]))

        self.tree_plates.delete(*self.tree_plates.get_children())
        table_data = self.db.get_table_data("Plates")
        for row in table_data:
            self.tree_plates.insert("", "end", values=row)

    def callback_plate_actions(self):
        if self.bool_plate_actions:
            return

        self.bool_plates = False
        self.bool_slot = False
        self.bool_plate_actions = True
        self.bool_wells = False
        self.current_tree.grid_remove()
        self.current_tree = self.tree_plate_actions
        self.current_tree.grid()
        for i in range(4):
            self.reset_button_state(self.button_list[i])
        self.button_plate_actions.configure(bootstyle="light")

        self.update_entry_fields(self.db.get_columns_from_table(self.tables[0]))

        self.tree_plate_actions.delete(*self.tree_plate_actions.get_children())
        table_data = self.db.get_table_data("PlateActions")
        for row in table_data:
            self.tree_plate_actions.insert("", "end", values=row)

    def callback_wells(self):
        if self.bool_wells:
            return

        self.bool_wells = True
        self.bool_plates = False
        self.bool_slot = False
        self.bool_plate_actions = False
        self.current_tree.grid_remove()
        self.current_tree = self.tree_wells
        self.current_tree.grid()
        for i in range(4):
            self.reset_button_state(self.button_list[i])
        self.button_wells.configure(bootstyle="light")

        self.update_entry_fields(self.db.get_columns_from_table(self.tables[2]))

        self.tree_wells.delete(*self.tree_wells.get_children())
        table_data = self.db.get_table_data("Wells")
        for row in table_data:
            self.tree_wells.insert("", "end", values=row)

    def reset_button_state(self, button: ttk.Button):
        button.configure(bootstyle = "Neutral Button")

    def update_entry_fields(self, columns: list[str]):

        for widget in self.entry_widgets:
            widget.destroy()

        self.entry_widgets.clear()

        for i in range(10):
            self.frame_entry.columnconfigure(i, weight=0)

        for index, col in enumerate(columns):
            self.frame_entry.columnconfigure(index, weight = 1)
            entry = ttk.Entry(self.frame_entry, width=15)
            entry.grid(row = 0, column=index, sticky="nsew", ipady = 8)
            self.entry_widgets.append(entry)

    def save_entry(self):
        if not self.current_tree:
            return

        # Zuordnung: TreeView → Tabellenname
        tree_to_table = {
            self.tree_plate_actions: self.tables[0],
            self.tree_plates: self.tables[1],
            self.tree_wells: self.tables[2],
            self.tree_slots: self.tables[4],
        }

        table_name = tree_to_table.get(self.current_tree)
        if not table_name:
            return

        # Spaltennamen holen & Eingabewerte erfassen
        columns = self.db.get_columns_from_table(table_name)
        values = [entry.get() for entry in self.entry_widgets]
        data = dict(zip(columns, values))

        try:
            if table_name == "Plates":

                format = data.get("Format", "6x8")
                if not re.fullmatch(r"\d+x\d+", format):
                    raise ValueError(f"Invaild format: '{format}' (expected example '6x8')")

                wells = int(data.get("Wells", 0))
                rows, cols = map(int, format.lower().split("x"))
                expected_wells = rows * cols
                if wells != expected_wells:
                    wells = expected_wells


                self.db.register_plate_with_wells(
                    plate_id=data["PlateId"],
                    owner=data.get("Owner", ""),
                    location=int(data["Location"]),
                    wells=wells,
                    format=format
                )

            elif table_name == "Slots":
                self.db.register_slot(
                    slot=int(data["SlotNumber"]),
                    stacker=int(data["Stacker"]),
                    role=int(data["Role"]),
                    occupied=int(data["Occupied"]),
                    assigned_id=data["AssignedPlatePlateId"],
                    current_id=data["CurrentPlateId"]
                )

            elif table_name == "Wells":
                self.db.register_well(
                    plate_id=data["plateID"],
                    location=data["location"],
                    mediachange=bool(int(data.get("mediachange", 0))),
                    is_media=bool(int(data.get("is_media", 0)))
                )

            elif table_name == "PlateActions":
                self.db.write_plate_action(
                    time_stamp=data["Timestamp"],
                    plate_id=data["PlateId"],
                    action=data["Action"]
                )

            # Erfolg: Zeile in TreeView einfügen
            self.current_tree.insert("", "end", values=values)

        except Exception as e:
            print(f"Fehler beim Speichern in '{table_name}': {e}")

